#!/bin/bash

# Initial Packager: Filippo "NeverBorn" Mesirca (filippo.mesirca@gmail.com)
# Maintainer : Son|c (emmanuel.bailleul@cegetel.net)
# Sources location : 
# http://us5.samba.org/samba/ftp/stable/samba-3.0.30.tar.gz
#
# Heavily based on Slackware-10.2 samba slackbuild

# program name/version
softversion='3.0.30'
softname1='samba'
softname2='sambaclient'

# architecture/package version
arch='i486'
cpu='i686'
packageversion1='54.1'
packageversion2='54.1'

# parameters needed for the build process
buildir=$(pwd)
package1="$softname1-$softversion-$arch-$packageversion1"
package2="$softname2-$softversion-$arch-$packageversion2"

# prepare the build result directory
dest1="$buildir/$package1"
dest2="$buildir/$package2"
rm -rf $dest1 $dest2
rm -f $package1.* $package2.*
mkdir -p $dest1
mkdir -p $dest2
mkdir -p $dest1/install
mkdir -p $dest1/usr/src/$softname1-$softversion
mkdir -p $dest2/install
mkdir -p $dest2/usr/src/$softname2-$softversion

# make the slack-desc files
cat <<EOF > $dest1/install/slack-desc
     |-----handy-ruler------------------------------------------------------|
     samba: Samba  (SMB server)
samba:
samba: Samba is an SMB file and print server for SMB and CIFS clients. It
samba: allows you to make file space or printers on a Samba host available
samba: any PCs running SMB clients (such as PCs running Windows).
samba: If you have any SMB servers (such as Windows NT/2K Server), you may
samba: be able to replace them by or supplement them with Samba. One of
samba: Samba's big strengths is integration, so you can use it to tie
samba: together your Linux hosts and Windows PC clients.
samba:
samba: 
EOF
cat <<EOF > $dest2/install/slack-desc
           |-----handy-ruler------------------------------------------------------|
sambaclient: Sambaclient (SMB client)
sambaclient:
sambaclient: Client for the SMB file and print sharing protocol
sambaclient:
sambaclient: One of Samba's big strengths is integration, so you can use it
sambaclient: to tie together you Linux hosts and Windows PC clients.
sambaclient:
sambaclient:
sambaclient:
sambaclient:
sambaclient: 
EOF

# extract sources
srcdir="samba-$softversion"
cd $buildir
rm -rf $srcdir
tar zxvf $srcdir.tar.gz
cd $srcdir

# apply a security patch
#cat $buildir/samba-3.0.21-CAN-2006-1059.patch | patch -p1

# set correct permissions
chown -R root.root . 
find . -perm 777 -exec chmod 755 {} \;
find . -perm 775 -exec chmod 755 {} \;
find . -perm 666 -exec chmod 644 {} \;
find . -perm 664 -exec chmod 644 {} \;

cd source
 
# configure package for compilation
export CFLAGS="-O2 -march=$arch -mtune=$cpu"
export CXXFLAGS="-O2 -march=$arch -mtune=$cpu"
./configure \
   --prefix=/usr \
   --sysconfdir=/etc \
   --localstatedir=/var \
   --enable-static=no \
   --enable-shared=yes \
   --enable-cups \
   --with-fhs \
   --with-privatedir=/etc/samba/private \
   --with-lockdir=/var/cache/samba \
   --with-swatdir=/usr/share/swat \
   --with-configdir=/etc/samba \
   --with-mandir=/usr/man \
   --with-logfilebase=/var/log/samba \
   --with-automount \
   --with-smbmount \
   --with-syslog \
   --with-quotas \
   --with-utmp \
   --with-libsmbclient \
   --with-ldap=no \
   --with-winbind
   
# compile package (parallel compilation is not used: it seems to break
# things)
make

# install package to the target directory
make install DESTDIR=$dest1

# Install libnss_win* libraries:
mkdir -p $dest1/lib
cp -a nsswitch/libnss_winbind.so $dest1/lib/libnss_winbind.so.2
cp -a nsswitch/libnss_wins.so $dest1/lib/libnss_wins.so.2
( cd $dest1/lib
  ln -sf libnss_winbind.so.2 libnss_winbind.so
  ln -sf libnss_wins.so.2 libnss_wins.so
)
# Sets the correct symlink
mkdir -p $dest2/sbin
( cd $dest2/sbin
  rm -f mount.smbfs
  ln -sf /usr/bin/smbmount mount.smbfs
  chown -R root.root .
  rm -f $dest1/sbin/mount.smbfs
)

# Make sure libsmbclient gets installed
( cd $dest1/usr/lib/samba
  rm -f libsmbclient.a
  mv libsmbclient.so ../libsmbclient.so.0.0
  cd ..
  rm -f libsmbclient.so.0 libsmbclient.so libsmbclient.a
)
chown root.root $dest1/usr/include/libsmbclient.h
chmod 644 $dest1/usr/include/libsmbclient.h

cd $buildir/$srcdir/examples

sed -i 's/log\ file\ =\ \/usr\/local\/samba\/var\/log/log\ file\ =\ \/var\/log\/samba/' smb.conf.default
sed -i 's/^\ \ \ path\ =\ \/usr\/spool\/samba/\ \ \ path\ =\ \/var\/spool\/samba/' smb.conf.default
#zcat $buildir/smb.conf.default.diff.gz | patch -p1
cp -a smb.conf.default $dest1/etc/samba/smb.conf-sample
cp -a smb.conf.default $dest1/etc/samba/smb.conf.new

# copy the doc files
mkdir -p $dest1/usr/doc/$softname1-$softversion
cd $buildir/$srcdir
cp -a \
   COPYING MAINTAINERS Manifest README Read-Manifest-Now Roadmap WHATSNEW.txt docs docs-xml \
   examples packaging $dest1/usr/doc/$softname1-$softversion

rm -rf $dest1/usr/doc/$softname1-$softversion/docs/htmldocs \
       $dest1/usr/doc/$softname1-$softversion/docs/manpages
( cd $dest1/usr/doc/$softname1-$softversion/docs
  ln -sf /usr/share/swat/help htmldocs
  ln -sf /usr/share/swat/help/using_samba using_samba
)
rm -f $dest1/usr/doc/$softname1-$softversion/docs/*.pdf
rm -rf $dest1/usr/doc/$softname1-$softversion/docs/docbook

find $dest1/usr/doc -type d -exec chmod 755 {} \;
find $dest1/usr/doc -type f -exec chmod 644 {} \;

# put the initialization script in $dest1/etc/rc.d/ 
mkdir -p $dest1/etc/rc.d

echo "#!/bin/sh
#
# /etc/rc.d/rc.samba
#
# Start/stop/restart the Samba SMB file/print server.
#
# To make Samba start automatically at boot, make this
# file executable:  chmod 755 /etc/rc.d/rc.samba
#

samba_start() {
  if [ -x /usr/sbin/smbd -a -x /usr/sbin/nmbd -a -r /etc/samba/smb.conf ]; then
    echo \"Starting Samba:  /usr/sbin/smbd -D\"
    /usr/sbin/smbd -D
    echo \"                 /usr/sbin/nmbd -D\"
    /usr/sbin/nmbd -D
  fi
}

samba_stop() {
  killall -s 9 smbd nmbd
}

samba_restart() {
  samba_stop
  sleep 2
  samba_start
}

case \"\$1\" in
'start')
  samba_start
  ;;
'stop')
  samba_stop
  ;;
'restart')
  samba_restart
  ;;
*)
  # Default is \"start\", for backwards compatibility with previous
  # Slackware versions.  This may change to a 'usage' error someday.
  samba_start
esac" > $dest1/etc/rc.d/rc.samba.new

#zcat $buildir/rc.samba.gz > $dest1/etc/rc.d/rc.samba.new
chmod 644 $dest1/etc/rc.d/rc.samba.new
chown root.root $dest1/etc/rc.d/rc.samba.new

# strip executables 
( cd $dest1
   find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
   find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip --strip-unneeded 2> /dev/null
)
# gzip man pages
gzip -9 $dest1/usr/man/*/*

# put the ZenWalk build and other needed files in the right directory
cp -a $buildir/build-$softname1.sh $dest1/usr/src/$softname1-$softversion
#cp -a $buildir/rc.samba.gz $dest1/usr/src/$softname1-$softversion
#cp -a $buildir/smb.conf.default.diff.gz $dest1/usr/src/$softname1-$softversion
#cp -a $buildir/doinst* $dest1/usr/src/$softname1-$softversion
chown -R root.root $dest1/usr/src/$softname1-$softversion

# change files ownerships for the $dest1/usr/bin and $dest1/usr/sbin
#chown -R root.root $dest1/usr/bin
#chown -R root.root $dest1/usr/sbin

# package sambaclient:
mkdir -p $dest2/etc
mv $dest1/etc/samba $dest2/etc
chmod 644 $dest2/etc/samba/smb.conf*
mkdir -p $dest2/usr/bin
cd $dest1/usr/bin
mv \
   findsmb nmblookup rpcclient smbcacls smbclient smbget smbmnt smbmount smbspool smbtar smbtree smbumount \
   $dest2/usr/bin
#mkdir -p $dest2/usr/doc/$softname2-$softversion
#cd $dest1/usr/doc/$softname1-$softversion
#cp -a \
#   COPYING Manifest Read-Manifest-Now README Roadmap WHATSNEW.txt \
#   $dest2/usr/doc/$softname2-$softversion
mkdir -p $dest2/usr/include
mv $dest1/usr/include/libsmbclient.h $dest2/usr/include
mkdir -p $dest2/usr/lib
mv $dest1/usr/lib/libsmbclient.so.0.0 $dest2/usr/lib
mkdir -p $dest2/usr/lib/samba
cd $dest2/usr/lib/samba
ln -sf ../libsmbclient.so.0.0 libsmbclient.so
cd $dest2/usr/lib/
ln -sf libsmbclient.so.0.0 libsmbclient.so.0
ln -sf libsmbclient.so.0.0 libsmbclient.so
mkdir -p $dest2/usr/man/man1
mkdir -p $dest2/usr/man/man5
mkdir -p $dest2/usr/man/man7
mkdir -p $dest2/usr/man/man8
( cd $dest1/usr/man/man1
  mv \
     findsmb.1.gz nmblookup.1.gz rpcclient.1.gz smbcacls.1.gz smbclient.1.gz smbget.1.gz smbtar.1.gz smbtree.1.gz \
     $dest2/usr/man/man1
)
( cd $dest1/usr/man/man5
  mv \
     smbgetrc.5.gz \
     $dest2/usr/man/man5
)
( cd $dest1/usr/man/man7
  mv \
     libsmbclient.7.gz \
     $dest2/usr/man/man7
)
( cd $dest1/usr/man/man8
  mv \
     mount.cifs.8.gz smbmnt.8.gz smbmount.8.gz smbspool.8.gz smbumount.8.gz umount.cifs.8.gz \
     $dest2/usr/man/man8
)
cp -a $dest1/usr/src/$softname1-$softversion/* $dest2/usr/src/$softname2-$softversion

# compile mount.cifs and umount.cifs (thanks to Gary Major for pointing
# this out). For the compilation options passed to gcc, I referred to
# the CFLAGS, LDFLAGS and LIBS options in
# $buildir/$srcdir/source/Makefile after the ./configure script had been
# executed.
cd $buildir/$srcdir/source/client
gcc -Wl,-rpath,/usr/lib -lcrypt -lresolv -lnsl -ldl -O2 -march=$arch -mtune=$cpu mount.cifs.c -o mount.cifs
gcc -Wl,-rpath,/usr/lib -lcrypt -lresolv -lnsl -ldl -O2 -march=$arch -mtune=$cpu umount.cifs.c -o umount.cifs
strip --strip-unneeded mount.cifs
strip --strip-unneeded umount.cifs
mkdir -p $dest2/usr/sbin
cp -a mount.cifs umount.cifs $dest2/usr/sbin

# Create doinst.sh files
echo "#!/bin/sh
config() {
  NEW=\"\$1\"
  OLD=\"\`dirname \$NEW\`/\`basename \$NEW .new\`\"
  # If there's no config file by that name, mv it over:
  if [ ! -r \$OLD ]; then
    mv \$NEW \$OLD
  elif [ \"\`cat \$OLD | md5sum\`\" = \"\`cat \$NEW | md5sum\`\" ]; then # toss the redundant copy
    rm \$NEW
  fi
  # Otherwise, we leave the .new copy for the admin to consider...
}
config etc/rc.d/rc.samba.new
" > $dest1/install/doinst.sh

echo "#!/bin/sh
config() {
  NEW=\"\$1\"
  OLD=\"\`dirname \$NEW\`/\`basename \$NEW .new\`\"
  # If there's no config file by that name, mv it over:
  if [ ! -r \$OLD ]; then
    mv \$NEW \$OLD
  elif [ \"\`cat \$OLD | md5sum\`\" = \"\`cat \$NEW | md5sum\`\" ]; then # toss the redundant copy
    rm \$NEW
  fi
  # Otherwise, we leave the .new copy for the admin to consider...
}
config etc/samba/smb.conf.new
" > $dest2/install/doinst.sh

#------------------( Create The *.dep Files )
cd $dest1
requiredbuilder -z $dest1 > $buildir/$package1.dep
echo ",sambaclient" >> $buildir/$package1.dep
cd $dest2
requiredbuilder -z $dest2 > $buildir/$package2.dep

# Make packages
cd $dest1
makepkg -l y -c n $package1.tgz
mv $package1.tgz $buildir
cd $dest2
makepkg -l y -c n $package2.tgz
mv $package2.tgz $buildir

# computes the md5sum signature for the package
cd $buildir
md5sum $package1.tgz > $package1.md5
md5sum $package2.tgz > $package2.md5


